// *******************************************************************
// File EbApiBase.h        The most basic OS functions and OS messages
// *******************************************************************
#ifndef _EbApiBase_h_
#define _EbApiBase_h_

#include "EbTypedefs.h"
#include "EbTime.h"

extern DWORD  _LastOsError;                     // To be set by the OS

// These can be set by Users:
extern LPCWSTR PopUpCaption;
extern void (*_ProcessMessagesFunction)(DWORD mode);

// *******************************************************************
// The most basic API functions
// *******************************************************************

  RETOK  _KillHandle(HANDLE* ph);
  RETERR _RefreshLastOsError();

  inline void  _ClearLastOsError(){_LastOsError = 0;}
  inline void  _SetLastOsError(DWORD err){_LastOsError = err;}
  inline DWORD _GetLastOsError(){return _LastOsError;}

// *******************************************************************
// Simplest message functions
// *******************************************************************

// Tendentially, at least, one should prefer the Unicode versions
// today, but the non-Unicode ones must be maintained as a legacy.

  void  _Beep(MsgTYPE type);

  inline void _SetPopUpCaption(LPCWSTR caption)
                              {PopUpCaption = caption;}

  LONG  _Message(MsgTYPE type,LPCSTR  text,LPCSTR  caption = NULL);
  LONG  _MessageW(MsgTYPE type,LPCWSTR text,LPCWSTR caption = NULL);

  LONG  _PopUp(MsgTYPE type,LPCSTR  format,
               LPCSTR  txt1=NULL,LPCSTR  txt2=NULL,LPCSTR  txt3=NULL);
  LONG  _PopUpW(MsgTYPE type,LPCWSTR format,
               LPCWSTR txt1=NULL,LPCWSTR txt2=NULL,LPCWSTR txt3=NULL);

// *******************************************************************
// Basic time functions
// *******************************************************************

  DWORD _TimeInMs();
  void  _GetSystemTimeAsFileTime(EbFILETIME* ftime);
  void  _GetLocalTime(EbSYSTEMTIME* stime);
  DWORD _GetTimeFormat(EbSYSTEMTIME* stime,LPSTR bfr,LONG bfrsize);
  DWORD _GetTimeFormatW(EbSYSTEMTIME* stime,LPWSTR bfr,LONG bfrsize);
  DWORD _GetDateFormat(EbSYSTEMTIME* stime,LPSTR bfr,LONG bfrsize);
  DWORD _GetDateFormatW(EbSYSTEMTIME* stime,LPWSTR bfr,LONG bfrsize);

  LPSTR  _AppendTime(LPSTR* pdest);
  LPWSTR _AppendTimeW(LPWSTR* pdest);
  LPWSTR _AppendDateW(LPWSTR* pdest);
  LPSTR  _AppendDate(LPSTR* pdest);
  LPSTR  _AppendDateAndTime(LPSTR* pdest,
  	     LPCSTR sep1=" ",LPCSTR sep2=" ",LPCSTR text=NULL);
  LPWSTR _AppendDateAndTime(LPWSTR* pdest,
             LPCWSTR sep1=L" ",LPCWSTR sep2=L" ",LPCWSTR text=NULL);

  void  _DeadWaitInMs(DWORD delay);
  void  _LiveWaitInMs(DWORD delay,LONG mode);
  void  _DeadWaitInMicroSec(DWORD microseconds);

// -------------------------------------------------------------------
#endif


