// *******************************************************************
// File EbBasicLib.h            	  	  Basic library header
// *******************************************************************
// Copyright @2014 by Stan Sykora and Extra Byte
#ifndef _EbBasicLib_h_
#define _EbBasicLib_h_

// This header does not refer to any OS API,
// nor to QT or C++ Builder.

#include "EbTypedefs.h"
#include "EbCpuAsm.h"
#include <stdio.h>

// GroomStr function flags (default is 0):
// -------------------------------------------------------------------
// a) Low WORD: Character grooming flags:
#define GRM_DONOTHING	 0x00008000 // Do nothing
#define GRM_LEAVEBIT7	 0x00000001 // Don't strip bit7
#define GRM_LEAVELEADER  0x00000002 // Leave leading  whitespace
#define GRM_LEAVETRAILER 0x00000004 // Leave trailing whitespace
#define GRM_LEAVEEXTRA   0x00000008 // Leave extra SPACES in the body
#define GRM_LEAVECTRL	 0x00000010 // Don't replace chars<0x20 by SPACE
#define GRM_LEAVECR	 0x00000020 // Leave Carriage Return
#define GRM_LEAVELF	 0x00000040 // Leave Line Feed
#define GRM_LEAVETB	 0x00000080 // Leave Tabulator
#define GRM_LEAVEALL	 0x000000ff // Leave all
				    // Conversions:
#define GRM_UCASE	 0x00000100 // Convert to upper case
#define GRM_LCASE	 0x00000200 // Convert to lower case
// b) High word: Special line-grooming flags:
#define LGRM_NOCOMMENT   0x00010000 // Strip comment

// Number testing flags:
// -------------------------------------------------------------------
#define NUM_OVERFLOW	   0x00000001 // Overflow (abs.value too large)
#define NUM_INTOVERFLOW	   0x00000002 // Integer overflow  (> 2147483647)
#define NUM_INTUNDERFLOW   0x00000004 // Integer underflow (<-2147483647)
#define NUM_FRACTIONAL     0x00000008 // Fractional part in integer
#define NUM_ILLEGALCHAR    0x00000010 // Illegal character encountered
#define NUM_UNDEFINED      0x00000020 // No valid digit in source
#define NUM_DWORDOVERFLOW  0x00000040 // Dword overflow (>4294967295)
#define NUM_DWORDUNDERFLOW 0x00000080 // Dword underflow (<0)
#define NUM_ABOVELIMIT     0x00000100 // Reserved for User
#define NUM_BELOWLIMIT     0x00000200 // Reserved for User

#define NUM_BADDOUBLE (NUM_OVERFLOW|NUM_ILLEGALCHAR|NUM_UNDEFINED)
#define NUM_BADLONG   (NUM_OVERFLOW|NUM_INTOVERFLOW|NUM_INTUNDERFLOW| \
		       NUM_FRACTIONAL|NUM_ILLEGALCHAR|NUM_UNDEFINED)
#define NUM_BADDWORD  (NUM_OVERFLOW|NUM_DWORDOVERFLOW|NUM_DWORDUNDERFLOW| \
		       NUM_FRACTIONAL|NUM_ILLEGALCHAR|NUM_UNDEFINED)

// ===================================================================
// Exception handling functions (experimental)
// ===================================================================

DWORD EbFUNCTION Except(void(*uFuck)(void*),void* userData);

// ===================================================================
// Memory functions
// ===================================================================

DWORD EbFUNCTION LargestAllocatableBlock();
void* EbFUNCTION MemAlloc(DWORD size);
void  EbFUNCTION MemKill(void** pmem);

void*   CopyMem(void* destination,const void* source,DWORD bytes);
void*   ZeroMem(void* destination,DWORD bytes);
void*   FillMemBytes(void* destination,DWORD bytes,BYTE fill);
void*   FillMemStr(void* destination,LPCSTR str);
void*   FillMemNetStr(void* destination,LPCSTR str);

// ===================================================================
// Character functions
// ===================================================================

inline char UCaseChr(char ch)
       {return ((ch>='a')&&(ch<='z'))?((char)(ch-32)):ch;}

inline char LCaseChr(char ch)
       {return ((ch>='A')&&(ch<='Z'))?((char)(ch+32)):ch;}

inline LONG HexCharToDigit(char ch)
       {return (ch>='A'?(LONG)(ch-'A'+10):(LONG)(ch-'0'));}

inline char DigitToHexChar(DWORD dw)
       {return ((char)((dw&0xf)>=10?(dw&0xf)-10+'A':(dw&0xf)+'0'));}

inline char BitToChar(DWORD dw)
       {return ((char)(dw&1?'1':'0'));}

// ===================================================================
// A special class simplifying conversions between LPSTR and LPWSTR
// ===================================================================
// It is particularly useful when one might possibly need both
// versions of a string. Or maybe just the Unicode version,
// converted from an ASCI string, but without forgetting to
// disallocate it.

class cStrAW
//--------------------------------------------------------------------
{
public:
  LPSTR  A;     // Dynamically allocated ASCI string
  LPWSTR W;     // Dynamically allocated Unicode string
public:
  ~cStrAW();
  cStrAW();
  cStrAW(LPCSTR a);
  cStrAW(LPCWSTR w);
  void Set(LPCSTR a);
  void Set(LPCWSTR w);
};

// ===================================================================
// String functions
// ===================================================================

// Strings handling with direct allocation/reallocation
// -------------------------------------------------------------------

LPSTR   NewString(DWORD length);
LPWSTR  NewStringW(DWORD length);
LPSTR*  KillString(LPSTR* pStr);
LPWSTR* KillStringW(LPWSTR* pStr);

// Strings handling with implicit allocation/reallocation
// -------------------------------------------------------------------

LPSTR   MakeStr(DWORD length, SCHAR fill);
LPWSTR  MakeStrW(DWORD length, WCHAR fill);
LPSTR   CopyString(LPCSTR str);
LPWSTR  CopyStringW(LPCWSTR str);
LPWSTR  CopyStringAW(LPCSTR str);
LPSTR   CopyStringWA(LPCWSTR str);
LPSTR   CopyStrEx(LPCSTR str,DWORD extra=0,LPSTR* pExtra=NULL);
LPWSTR  CopyStrExW(LPCWSTR str,DWORD extra=0,LPWSTR* pExtra=NULL);
LPSTR   ReplaceStr(LPSTR* pStr,LPCSTR newStr);
LPWSTR  ReplaceStrW(LPWSTR* pStr,LPCWSTR newStr);
LPSTR   ReplaceStrEx
         (LPSTR* pStr,LPCSTR newStr,DWORD extra,LPSTR* pExtra);
LPWSTR  ReplaceStrExW
         (LPWSTR* pStr,LPCWSTR newStr,DWORD extra,LPWSTR* pExtra);
LPSTR   AppendStrings(LPSTR* pStr,LPCSTR addStr1,LPCSTR addStr2=NULL,
         LPCSTR addStr3=NULL,LPCSTR addStr4=NULL,LPCSTR addStr5=NULL);
LPWSTR  AppendStrings(LPWSTR* pStr,LPCWSTR addStr1,LPCWSTR addStr2=NULL,
         LPCWSTR addStr3=NULL,LPCWSTR addStr4=NULL,LPCWSTR addStr5=NULL);
LPWSTR  AppendStrings(LPWSTR* pStr,LPCSTR addStr1,LPCSTR addStr2=NULL);
LPSTR   AppendLastChrIfAbsent(LPSTR* pStr,char Chr);
LPWSTR  AppendLastChrIfAbsentW(LPWSTR* pStr,wchar_t Chr);
LPSTR   NextAlphaCount(LPSTR* alphacount);
LPWSTR  NextAlphaCountW(LPWSTR* alphacount);

// String buffers management without any allocation
// -------------------------------------------------------------------

LPSTR   LoadStr(LPSTR buffer,LPCSTR str);
LPWSTR  LoadStrW(LPWSTR buffer,LPCWSTR str);
LPSTR   SetCharsInStr(LPSTR str,DWORD nch,char ch);
LPWSTR  SetCharsInStr(LPWSTR str,DWORD nch,wchar_t ch);

// String query functions
// -------------------------------------------------------------------

LPSTR   pEndOfStr(LPCSTR str);
LPWSTR  pEndOfStrW(LPCWSTR str);
DWORD   StrLength(LPCSTR str);
DWORD   StrLengthW(LPCWSTR str);
DWORD   isEmptyStr(LPCSTR str);
DWORD   isEmptyStrW(LPCWSTR str);
DWORD   CharCount(LPCSTR str,char chr);
DWORD   CharCountW(LPCWSTR str,wchar_t chr);
DWORD   LineCount(LPCSTR str);
DWORD   LineCountW(LPCWSTR str);
char    FirstNonBlankChar(LPCSTR);
wchar_t FirstNonBlankChar(LPCWSTR);
LPSTR   ChrInStrFirst(LPCSTR str, char Chr);
LPWSTR  ChrInStrFirstW(LPCWSTR str,wchar_t Chr);
LPSTR   ChrInStrLast(LPCSTR str, char Chr);
LPWSTR  ChrInStrLastW(LPCWSTR str,wchar_t Chr);
LPSTR   FindFirstOfChars(LPCSTR str,LPCSTR chars,char* chr);
LPWSTR  FindFirstOfCharsW(LPCWSTR str,LPCWSTR chars,wchar_t* chr);
LPSTR   FindLastOfChars(LPCSTR str,LPCSTR chars,char* chr);
LPWSTR  FindLastOfCharsW(LPCWSTR str,LPCWSTR chars,wchar_t* chr);
RETOK   StrStartsWith(LPCSTR str,LPCSTR header);
RETOK   StrStartsWithW(LPCWSTR str,LPCWSTR header);

// Comparing two strings
// -------------------------------------------------------------------

LONG    CompareStrings(LPCSTR str1,LPCSTR str2);
LONG    CompareStringsW(LPCWSTR str1,LPCWSTR str2);
LONG    CmpStrings(LPCSTR str1,LPCSTR str2,DWORD flags=0);
LONG    CmpStringsW(LPCWSTR str1,LPCWSTR str2,DWORD flags=0);
DWORD   StrGroomedEq(LPSTR arg1,LPSTR arg2);
DWORD   StrGroomedGt(LPSTR arg1,LPSTR arg2);
DWORD   StrGroomedLt(LPSTR arg1,LPSTR arg2);

// String in-place management functions
// -------------------------------------------------------------------

LPSTR   ReplaceChrByChr(LPSTR str,char OrigChr,char NewChr);
LPWSTR  ReplaceChrByChrW(LPWSTR str,wchar_t OrigChr,wchar_t NewChr);
LPSTR   ReplaceControls(LPSTR str);
LPWSTR  ReplaceControlsW(LPWSTR str);
LPSTR   GroomStr(LPSTR str,DWORD flags=0);
LPWSTR  GroomStrW(LPWSTR str,DWORD flags=0);

inline  LPSTR UCaseStr(LPSTR str)
	      {return GroomStr(str,GRM_LEAVEALL|GRM_UCASE);}
inline  LPWSTR UCaseStrW(LPWSTR str)
	      {return GroomStrW(str,GRM_LEAVEALL|GRM_UCASE);}
inline  LPSTR LCaseStr(LPSTR str)
	      {return GroomStr(str,GRM_LEAVEALL|GRM_LCASE);}
inline  LPWSTR LCaseStrW(LPWSTR str)
	      {return GroomStrW(str,GRM_LEAVEALL|GRM_LCASE);}

void    CheckBackslashTerminator(LPSTR str);
void    CheckBackslashTerminatorW(LPWSTR str);


// SubString searching functions
// -------------------------------------------------------------------

void    BoyerMooreSkipTable(LPCSTR sub,DWORD lsub,DWORD* skip);
void    BoyerMooreSkipTableEx(LPCSTR sub, DWORD lsub,
	                      fSEARCH flags,DWORD* skip);
LPSTR   LocateSubString(LPCSTR str,LPCSTR sub,
	      DWORD lstr=0,DWORD lsub=0,DWORD* skip=NULL);
LPSTR   LocateSubStringEx(LPCSTR str,LPCSTR sub,
	      DWORD lstr=0,DWORD lsub=0,
              fSEARCH flags=sDefault,DWORD* skip=NULL);
RETOK   DeleteSubString(LPCSTR str,LPCSTR sub,
	      DWORD lstr=0,DWORD lsub=0,DWORD* skip=NULL);

// Character-delimited string lists
// -------------------------------------------------------------------

LPSTR  SetItemSkipPtr(LPSTR* pitem,LPSTR* pstr,
	      char delimiter=Chr_COMMA,
              DWORD flags=GRM_LEAVEBIT7|GRM_LEAVECTRL|GRM_LEAVEEXTRA);
DWORD  NumberOfStrItems(LPCSTR str,char delimiter=Chr_COMMA);
DWORD  WhichItemInStr(LPCSTR item,LPCSTR string,
	      char delimiter=Chr_COMMA,DWORD ignorecase=0,
	      DWORD flags=GRM_LEAVEBIT7|GRM_LEAVECTRL|GRM_LEAVEEXTRA);
LPSTR  AppendStringListItem
	     (LPSTR* plist, LPCSTR item, LPSTR seps=Str_CR);
RETOK  DeleteStringListItem
	     (LPSTR list, LPCSTR item, LPSTR seps=Str_CR);
LPSTR  FindStringListItem
	     (LPSTR list, LPCSTR item, LPSTR seps=Str_CR);

// String lists with multiple separators
// -------------------------------------------------------------------

LPSTR  NthItemInStr(LPCSTR str,DWORD n,LPCSTR separators);

// Item-Value string lists
// -------------------------------------------------------------------

LPSTR  AppendItemFromList(LPSTR* pdest,LPCSTR item,
             LPCSTR list,char ssep,char sterm);

// ===================================================================
// Numeric conversions
// ===================================================================
// Globals:

extern  LONG NUM_PRECISION_g;			   // default for gcvt

// Numeric strings tests

RETERR  TestDoubleText(LPSTR text,
             double min=-1e300,double max=1e300,double* presult=NULL);
RETERR  TestDwordText(LPSTR text,
             DWORD min=0,DWORD max=0xffffffff,DWORD* presult=NULL);

// String to number

double  StrToD(LPCSTR sval,LPSTR* pendp);
double	StringToDouble
             (LPCSTR sval,DWORD* pflags=NULL,LPSTR* pendp=NULL);
LONG 	StringToLong(LPCSTR sval,DWORD* pflags=NULL);
DWORD 	StringToDword(LPCSTR sval,DWORD* pflags=NULL);
LONG	DecOrHexStringToLong(LPCSTR sval,DWORD* pflags=NULL);
DWORD   HexStringToDword(LPCSTR sval,DWORD* pflags=NULL);

// Number to string

LPSTR   DwordToAsci(DWORD dw,LPSTR bfr,DWORD radix);
LPSTR   LongToAsci(LONG l,LPSTR bfr,DWORD radix);
LPSTR   DoubleToAsci(double val,LPSTR bfr,DWORD ndigits=0);
LPSTR	DwordToNewString(DWORD val,DWORD radix=10);
LPSTR	LongToNewString(LONG val,DWORD radix=10);
LPSTR	DoubleToNewString(double val,LONG ndig=NUM_PRECISION_g);
DWORD   DoubleToAsciBuffer(const double f,LPSTR bfr,
             DWORD ndigits=NUM_PRECISION_g,DWORD radix=10);
LPSTR	AppendNumStr(LPSTR* pStr, double val,LPCSTR before=NULL,
             LPCSTR after=NULL,LONG ndig=NUM_PRECISION_g);
LPSTR   KMGByteBraDec(LPSTR bfr,DWORD dw,DWORD noextra=0);

// ===================================================================
// Number to number conversions and comparisons
// ===================================================================

double	NormalizeDouble(double val,LONG ndig=NUM_PRECISION_g);
LONG	DoubleToLong(double val,DWORD* pflags=NULL);
DWORD   DoubleToDword(double val,DWORD* pflags);
BOOL    lCnvTestLimits(REAL f,LONG* dest,
                LONG minval=0x80000001L,LONG maxval=0x7fffffffL);
BOOL    dwCnvTestLimits(REAL f,DWORD* dest,
                DWORD minval=0uL,DWORD maxval=0xffffffffuL);
double	SymmetricFloor(double f);   // For -2.3 returns -2.0, not -3.0
LONG	CompareDouble(const void* v1, const void* v2);

// ===================================================================
// String-based numeric calculus
// ===================================================================

LPSTR NormalizeStringDouble
        (LPSTR* psval,DWORD* pflags=NULL,LONG ndig=NUM_PRECISION_g);
DWORD StrNumEq(LPCSTR arg1,LPCSTR arg2);
DWORD StrNumGt(LPCSTR arg1,LPCSTR arg2);
DWORD StrNumLt(LPCSTR arg1,LPCSTR arg2);
void  StrNumAdd(LPSTR* pdest,LPCSTR arg);
void  StrNumSubtract(LPSTR* pdest,LPCSTR arg);
void  StrNumMultiply(LPSTR* pdest,LPCSTR arg);
DWORD StrNumDivide(LPSTR* pdest,LPCSTR arg);

//--------------------------------------------------------------------
#endif


