//********************************************************************
// File EbTypedefs.h  	                        Basic Extra Byte types
//********************************************************************
// Copyright @2013 by Stan Sykora and Extra Byte
#ifndef _EbTypedefs_h_
#define _EbTypedefs_h_

#define EbFUNCTION __declspec( dllexport )

// These typdefs are OS-independent but compatible with WIN32

typedef int			BOOL;

typedef void*			LPVOID;
typedef const void*		LPCVOID;
typedef char*			LPSTR;
typedef const char*		LPCSTR;

typedef wchar_t 		WCHAR;
typedef wchar_t* 		LPWSTR;
typedef const wchar_t*		LPCWSTR;

typedef unsigned char		BYTE;
typedef unsigned char		BOOLEAN;
typedef unsigned char*		LPBYTE;
typedef const unsigned char*    LPCBYTE;

typedef signed char     	SCHAR;
typedef signed char*    	LPSCHAR;
typedef const signed char*	LPCSCHAR;

typedef unsigned short		WORD;
typedef unsigned short*		LPWORD;
typedef const unsigned short* 	LPCWORD;

typedef signed short		SHORT;
typedef signed short*		LPSHORT;
typedef const signed short*	LPCSHORT;

typedef unsigned long		RETOK;
typedef unsigned long		RETERR;
typedef unsigned long		DWORD;
typedef unsigned long*		LPDWORD;
typedef unsigned long*		LPCDWORD;

typedef signed long     	LONG;
typedef signed long*		LPLONG;
typedef const signed long*	LPCLONG;

typedef double			REAL;
typedef double*			LPREAL;
typedef const double*		LPCREAL;

typedef long double     	LREAL;
typedef long double*     	LPLREAL;
typedef const long double*     	LPCLREAL;

#ifndef _WINDEF_   // This is the only item possibly conflicting Win32
typedef struct tagRECT
{
    LONG    left;
    LONG    top;
    LONG    right;
    LONG    bottom;
} RECT, *LPRECT;
#endif

// ------------------------------------------------------- Array types
// Note: For floating point numbers, use consistently the REAL type
// which makes it easy, if desired, to switch between single or
// double precision, or another type which might come up.
// -------------------------------------------------------------------

typedef DWORD* DWARR;        // DWORDs array
typedef DWORD* DWARRM;       // DWORDs square matrix
typedef DWORD* DWARRLT;      // DWORDs lower triangular matrix

typedef LONG*  LARR;         // LONGs array
typedef LONG*  LARRM;        // LONGs square matrix
typedef LONG*  LARRLT;       // LONGs lower triangular matrix

typedef REAL*  REALV;        // REALs array
typedef REAL*  REALM;        // REALs square matrix
typedef REAL*  REALT;        // REALs lower triangular matrix

// Note: all types of matrices are stored as linear REAL arrays
// Their structure is taken care of when calculating indices/sizes

typedef REAL** REALPV;       // Array of pointers to REALs
typedef REAL** REALPT;       // Triangular matrix of pointers to REALs

// ------------------------------------------- Function argument types
#define IN      // Input
#define OUT     // Output
#define INOUT   // Input & output

// ------------------------------------------------- Numeric constants
#ifndef NULL
#define NULL            0
#endif
#ifndef FALSE
#define FALSE           0
#endif
#ifndef TRUE
#define TRUE            1
#endif

#define NULLPTR		((void*)0)
#define NULLSTR		((LPSTR)0)
#define NULLDWORD      	0lu
#define NULLLPDWORD     ((DWORD*)0)
#define NULLLONG      	0L
#define NULLLPLONG      ((LONG*)0)
#define NULLDOUBLE     	0.
#define NULLLPDOUBLE    ((DOUBLE*)0)

#define MAXBYTE   	0xff
#define MAXWORD	  	0xffff
#define MAXDWORD	0xffffffff

#define MAXCHAR	  	0x7f
#define MAXSHORT  	0x7fff
#define MAXLONG	  	0x7fffffff

#define BIT_0	0x1
#define BIT_1	0x2
#define BIT_2	0x4
#define BIT_3	0x8
#define BIT_4	0x10
#define BIT_5	0x20
#define BIT_6	0x40
#define BIT_7	0x80
#define BIT_8	0x100
#define BIT_9	0x200
#define BIT_10	0x400
#define BIT_11	0x800
#define BIT_12	0x1000
#define BIT_13	0x2000
#define BIT_14	0x4000
#define BIT_15	0x8000
#define BIT_16	0x10000
#define BIT_17	0x20000
#define BIT_18	0x40000
#define BIT_19	0x80000
#define BIT_20	0x100000
#define BIT_21	0x200000
#define BIT_22	0x400000
#define BIT_23	0x800000
#define BIT_24	0x1000000
#define BIT_25	0x2000000
#define BIT_26	0x4000000
#define BIT_27	0x8000000
#define BIT_28	0x10000000
#define BIT_29	0x20000000
#define BIT_30	0x40000000
#define BIT_31	0x80000000

#define TwoTo8		BIT_8
#define TwoTo16		BIT_16
#define TwoTo24		BIT_24
#define TwoTo32		4294967296.

// Special characters
// -------------------------------------------------------------------
#define Chr_NULL  	(0)		// Null (z-string terminator)
#define Chr_TAB	  	(0x09)		// Tabulator (\t)
#define Chr_LF	  	(0x0A)		// LineFeed (\n)
#define Chr_CR	  	(0x0D)		// CarriageReturn (\r)
#define Chr_EOT	  	(0x1A)		// EndOfText
#define Chr_SPACE 	(0x20)		// Space
#define Chr_COMMA 	(',')		// Comma
#define Chr_BAR	  	('|')		// Vertical bar
#define Chr_APOSTROPHE	('\'')		// Apostrophe
#define Chr_QUOTE  	('\"')		// Double quote mark
#define Chr_COLON	(':')		// Colon
#define Chr_SEMICOLON	(';')		// Semicolon

// Elementary strings
// -------------------------------------------------------------------
#define Str_CR      	("\x0D") 	// CR
#define Str_LF      	("\x0A") 	// LF
#define Str_CRLF    	("\x0D\x0A")   	// CR+LF
#define Str_TAB	  	("\x09")        // Tabulator
#define Str_EOT	  	("\x1A")	// EndOfText
#define Str_SPACE 	("\x20")	// Space
#define Str_COMMA   	(",")        	// Comma
#define Str_BAR	  	("|")		// Vertical bar
#define Str_APOSTROPHE	("\'")		// Apostrophe
#define Str_QUOTE  	("\"")		// Double quote mark
#define Str_COLON	(":")		// Colon
#define Str_SEMICOLON	(";")		// Semicolon

// Colors
//--------------------------------------------------------------------
#define PaletteRGB(color) (0x02000000|(color))

#define colorPanelDark 		  PaletteRGB(0xa0a0a0)
#define colorPanel		  PaletteRGB(0xc0c0c0)
#define colorPanelNice		  PaletteRGB(0xd2d8e0)
#define colorPanelLight		  PaletteRGB(0xe0e0e0)
#define colorPanelFume		  PaletteRGB(0xe6e8ea)
#define colorPanelWarm		  PaletteRGB(0xebf0f4)

#define colorSilver		  PaletteRGB(0xc0c0c0)
#define colorSilverHeavy	  PaletteRGB(0xb0b0b0)
#define colorSilverHeavyWarm	  PaletteRGB(0xabb0b4)
#define colorSilverWarm		  PaletteRGB(0xbbc0c4)
#define colorSilverLight	  PaletteRGB(0xd0d0d0)
#define colorSilverLightWarm	  PaletteRGB(0xcbd0d4)
#define colorSilverFume	          PaletteRGB(0xe0e0e0)
#define colorSilverFumeWarm       PaletteRGB(0xdbe0e4)
#define colorSilverFumeLight      PaletteRGB(0xf0f0f0)
#define colorSilverFumeLightWarm  PaletteRGB(0xebf0f4)

#define colorHeader		  PaletteRGB(0xc8d0d8)
#define colorPrompt		  PaletteRGB(0xa8d8ff)

#define colorGray 		  PaletteRGB(0xa0a0a0)
#define colorGrayLight            PaletteRGB(0xf0f0f0)
#define colorGrayDark             PaletteRGB(0x808080)
#define colorGrayWarm 		  PaletteRGB(0x9ba0a4)

#define colorBlueLight		  PaletteRGB(0xfffff0)
#define colorAqua		  PaletteRGB(0xffffa0)
#define colorAquaLight		  PaletteRGB(0xffffe0)

#define colorYellow		  PaletteRGB(0xc0ffff)
#define colorYellowLight	  PaletteRGB(0xe0ffff)

#define colorGreenLight		  PaletteRGB(0xe8ffe8)
#define colorRedLight		  PaletteRGB(0xB0B0FF)
#define colorVioletLight	  PaletteRGB(0xfff0f0)
#define colorPinkLight		  PaletteRGB(0xe8e8ff)
#define colorInfoBk		  PaletteRGB(0xe0ffff)

//--------------------------------------------------------------------
// Message types
//--------------------------------------------------------------------
enum MsgTYPE{_INFO,_WARNING,_ERROR,_YESNO,_YESNOCANCEL};

//--------------------------------------------------------------------
// Windows-compatible stuff
//--------------------------------------------------------------------
typedef void* HANDLE;
#define INVALID_HANDLE  ((HANDLE)-1)
#define isHandleInvalid(handle) ((handle)==INVALID_HANDLE)
#define isHandleValid(handle) (!isHandleInvalid(handle))
#define isHandleInvalidZ(handle) ((!(handle))||((handle)==INVALID_HANDLE))
#define isHandleValidNZ(handle) (!isHandleInvalidZ(handle))

// *******************************************************************
// Macros
// *******************************************************************

#define mSwap(a,b,aux) {(aux)=(a);(a)=(b);(b)=(aux);}

#define mMaxIn2(a,b) (((a)<(b))?(b):(a))
#define mMaxIn3(a,b,c) mMaxIn2(mMaxIn2(a,b),c)
#define mMaxIn4(a,b,c,d) mMaxIn2(mMaxIn2(a,b),mMaxIn2(c,d))
#define mMinIn2(a,b) (((a)>(b))?(b):(a))
#define mMinIn3(a,b,c) mMinIn2(mMinIn2(a,b),c)
#define mMinIn4(a,b,c,d) mMinIn2(mMinIn2(a,b),mMinIn2(c,d))

// Use this in conjunction with tests on RETERR types
#define NoError(reterr) ((reterr)==0)
#define IsError(reterr) ((reterr)!=0)

// Use this in conjunction with tests on RETOK types
#define IsOk(retok)  (retok)
#define NotOk(retok) (!(retok))

// Data extraction (ptr is an abstract pointer)
#define Dword(i,ptr)        (((DWORD*)(ptr))[i])
#define Word(i,ptr)         (((WORD*)(ptr))[i])
#define Byte(i,ptr)         (((BYTE*)(ptr))[i])
#define Nibble(i,ptr)       ((Byte((i)>>1,ptr)>>(((i)&1)<<2))&0xf)
#define Bit(i,ptr)          (Byte((i)>>3,ptr)&(1<<(i)%8)?1:0)

// Bitflag functions to use with integer types:
#define BitsOn(x,y)         ((x)|=(y))
#define BitsOff(x,n)        ((x)&= ~(n))
#define BitsOffOn(x,y,n)    ((x)=(y)|((x)& ~(n)))
#define isBits(x,b)         (((x)&(b))==(b))
#define isAnyBit(x,b)       ((x)&(b))
#define isMaskedBits(x,b,m) (((x)&(m))==(b))

// Character ch functions:
#define isCapLetter(ch)     (((ch)>='A')&&((ch)<='Z'))
#define isSmallLetter(ch)   (((ch)>='a')&&((ch)<='z'))
#define isCSLetter(ch)      (isCapLetter(ch)||isSmallLetter(ch))
#define isDecDigit(ch)      (((ch)>='0')&&((ch)<='9'))
#define isHexDigit(ch)      (isDecDigit(ch)|| \
   ((((ch)>='A') && ((ch)<='F'))||(((ch)>='a')&&((ch)<='f'))))
#define isHexOrDecDigit(ch,hex)  (isDecDigit(ch)|| \
   ((hex)&&((((ch)>='A')&&((ch)<='F'))||(((ch)>='a')&&((ch)<='f')))))
#define isOctalDigit(ch)    (((ch)>='0')&&((ch)<='7'))
#define isBinaryDigit(ch)   (((ch)=='0')||((ch)=='1'))

// String functions:
#define strTrueOrFalse(ex)  ((ex)?"True":"False")
#define strTRUEOrFALSE(ex)  ((ex)?"TRUE":"FALSE")
#define strtrueOrfalse(ex)  ((ex)?"true":"false")
#define strYesOrNot(ex)     ((ex)?"Yes":"Not")
#define strYESOrNOT(ex)     ((ex)?"YES":"NOT")
#define stryesOrnot(ex)     ((ex)?"yes":"not")

// The following are usable also for wchar_t:
#define isVoidStr(str)      (((str)==NULL)||(*(str)==0))
#define isNotVoidStr(str)   (((str)!=NULL)&&(*(str)!=0))
#define FirstChar(str)      ((str)[0])
#define LastChar(str)       ((str)[StrLength(str)-1])
#define SkipWhiteSpace(ptr) \
        {while (*(ptr)&&(*(ptr)<=32)) (ptr)++;}
#define SkipTillWhiteSpace(ptr) \
        {while (*(ptr)&&(*(ptr)>32)) (ptr)++;}
#define SkipToEnd(ptr)      {while (*(ptr)) (ptr)++;}
#define SkipToChar(ptr,ch)  {while (*(ptr)&&(*(ptr)!=(ch))) (ptr)++;}
#define SkipBeyondChar(ptr,ch) \
        { while (*(ptr)&&(*(ptr)!=(ch))) (ptr)++; \
          if (*(ptr)) (ptr)++;}

                            // These two are used in pairs by parsers:
                // The buffer pointed-to by ptr must have write access
#define TerminateAtChar(ptr,ch) \
        {while(*(ptr)&&(*(ptr)!=ch))(ptr)++;*(ptr)=0;}
#define SkipToCharAndTerminate(ptr,ch,auxch) \
        {while(*(ptr)&&(*(ptr)!=ch))(ptr)++;auxch=*(ptr);*(ptr)=0;}
#define RestoreAndSkipTerminator(ptr,auxch) \
        {*(ptr)=auxch;if(auxch)ptr++;}

// BitsField handling:
#define dwBitsFieldSize(nflags)  (((nflags)+31)>>5)    // Dword fields
#define dwBitsFieldSet(pBF,n)    (pBF[(n)>>5]|=  (1<<((n)&0x1f)))
#define dwBitsFieldToggle(pBF,n) (pBF[(n)>>5]^=  (1<<((n)&0x1f)))
#define dwBitsFieldReset(pBF,n)  (pBF[(n)>>5]&= ~(1<<((n)&0x1f)))
#define wBitsFieldSize(nflags)   (((nflags)+15)>>4)     // Word fields
#define wBitsFieldSet(pBF,n)     (pBF[(n)>>4]|=  (1<<((n)&0xf)))
#define wBitsFieldToggle(pBF,n)  (pBF[(n)>>4]^=  (1<<((n)&0xf)))
#define wBitsFieldReset(pBF,n)   (pBF[(n)>>4]&= ~(1<<((n)&0xf)))
#define bBitsFieldSize(nflags)   (((nflags)+7)>>3)      // Byte fields
#define bBitsFieldSet(pBF,n)     (pBF[(n)>>3]|=  (1<<((n)&0x7)))
#define bBitsFieldToggle(pBF,n)  (pBF[(n)>>3]^=  (1<<((n)&0x7)))
#define bBitsFieldReset(pBF,n)   (pBF[(n)>>3]&= ~(1<<((n)&0x7)))

// Integer arithmetic functions (Little Endian):
#define isNotTwo2N(dwl)     ((dwl))&((dwl)-1))

// Matrices:
          // Linear index in a lower-triangular matrix with col<=row:
#define LTriIdx(r,c) ((((r)*((r)+1))>>1)+(c))
          // Dimension of the array holding a lower-triangular matrix:
#define LTriDim(size) (((size)*((size)+1))>>1)
          // Linear index in a rectangular matrix: columnwise
          // Note: colsize = maxrowindex + 1 = number of rows
#define MatIdx(row,col,colsize) (((col)*(colsize))+(row))

// Character buffers allocation
//--------------------------------------------------------------------

#define BigDynamicBuffer(name)    name = NewString(65535)
#define BigDynamicBufferW(name)   name = NewStringW(65535)
#define BigDynamicBufferSize      65535

#define DynamicBuffer(name)       name = NewString(8191)
#define DynamicBufferW(name)      name = NewStringW(8191)
#define DynamicBufferSize         8191

#define SmallDynamicBuffer(name)  name = NewString(1023)
#define SmallDynamicBufferW(name) name = NewStringW(1023)
#define SmallDynamicBufferSize    1023

#define BigStackBuffer(name)	  char name[1024]
#define BigStackBufferW(name)	  wchar_t name[1024]
#define BigStackBufferSize	  1023

#define StackBuffer(name)	  char name[256]
#define StackBufferW(name)	  wchar_t name[256]
#define StackBufferSize		  255

#define SmallStackBuffer(name)	  char name[64]
#define SmallStackBufferW(name)	  wchar_t name[64]
#define SmallStackBufferSize	  63

// Generif search flags
//--------------------------------------------------------------------

enum fSEARCH {                                  // Searched file type:
  sBinary =            0x8000,                // Ignores EOT character
  sSkipAfter =	       0x4000,	       // Used in LocateString in file
  sIgnoreBlankLines =  0x2000,     // Used in loading lines from files
  sIgnoreCase =        0x1000,                        // Search modes:
  sUseWildChar =       0x0800,          // Wild character = upper Word

  sIgnoreAllWhite =    0x0400,		       // White space handling
  sIgnoreExtraWhite =  0x0200,
  sAllWhiteEqual =     0x0100,
  sQwertyEqual =       0x0080,
  sQwertyMistype =     0x0040,

  sFromBeginning =     0x0001,                    // Search direction:
  sRewind =	       0x0001,		     // Same as sFromBeginning
  sFromEnd =   	       0x0002,
  sFromCurrentPos =    0x0000,           // from current pos. forwards
  sBackwards =         0x0003,		// from current pos. backwards

  sDefault =           0			      // Default flags
};

#include "EbErrorCodes.h"

//--------------------------------------------------------------------
#endif



